@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021-2023 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -18,6 +18,7 @@ logging every update_period seconds (and resetting the count)
 import enum
 import multiprocessing
 import multiprocessing.connection as mpc
+from logging import Formatter
 
 import pyDE1.config
 
@@ -34,18 +35,21 @@ class MQTTStatusText (enum.Enum):
     on_will = 'Died'
 
 
-def run_mqtt_outbound(config: pyDE1.config.Config,
+def run_mqtt_outbound(master_config: pyDE1.config.Config,
                       log_queue: multiprocessing.Queue,
                       outbound_pipe: mpc.Connection,
                       mode: OutboundMode):
 
+    pyDE1.config.config = master_config
+    from pyDE1.config import config
+
     import asyncio
     import json
     import logging
     import os
     import time
 
-    from collections import Callable
+    from typing import Callable
     from socket import gethostname
 
     import paho.mqtt.client as mqtt
@@ -81,17 +85,17 @@ def run_mqtt_outbound(config: pyDE1.config.Config,
     loop = asyncio.get_event_loop()
     loop.set_debug(True)
 
-    def on_shutdown_underway_cleanup():
+    async def cleanup_on_shutdown():
         logger.info("Watching for shutdown event")
-        sm.shutdown_underway.wait()
+        await sm.wait_for_shutdown_underway()
         logger.info("Shutting down MQTT client")
+        if mqtt_client.is_connected():
+            mqtt_client.disconnect()
         mqtt_client.loop_stop()
-        mqtt_client.disconnect()
         logger.info("Setting cleanup_complete")
         sm.cleanup_complete.set()
 
-    on_shutdown_wait_task = loop.run_in_executor(
-        None, on_shutdown_underway_cleanup)
+    loop.create_task(cleanup_on_shutdown())
 
     sm.attach_signal_handler_to_loop(sm.shutdown, loop)
 
@@ -204,7 +208,7 @@ def run_mqtt_outbound(config: pyDE1.config.Config,
     mqtt_client.enable_logger(client_logger)
 
     if config.mqtt.USERNAME is not None:
-        logger.info(f"Connecting with username '{config.mqtt.USERNAME}'")
+        logger.info(f"Connecting MQTT with username '{config.mqtt.USERNAME}'")
         mqtt_client.username_pw_set(
             username=config.mqtt.USERNAME,
             password=config.mqtt.PASSWORD
@@ -248,28 +252,48 @@ def run_mqtt_outbound(config: pyDE1.config.Config,
 
             item_json = outbound_pipe.recv()
             item_as_dict = json.loads(item_json)
-            topic = f"{config.mqtt.TOPIC_ROOT}/{item_as_dict['class']}"
-            mqtt_client.publish(
-                topic=topic,
-                payload=item_json,
-                qos=0,
-                # retain=True,  # Can cause client to always check if "current"
-                retain=False,
-                properties=None
-            )
+            if 'class' in item_as_dict.keys():  # Is an event payload
+                topic = f"{config.mqtt.TOPIC_ROOT}/{item_as_dict['class']}"
+                mqtt_client.publish(
+                    topic=topic,
+                    payload=item_json,
+                    qos=0,
+                    # retain=True,  # Can cause client to always check if "current"
+                    retain=False,
+                    properties=None
+                )
 
-            now = time.time()
-            try:
-                counts[item_as_dict['class']] += 1
-            except KeyError:
-                counts[item_as_dict['class']] = 1
-            if now - last_update > update_period:
-                logger.debug(counts)
-                counts = {}
-                last_update = now
+                now = time.time()
+                try:
+                    counts[item_as_dict['class']] += 1
+                except KeyError:
+                    counts[item_as_dict['class']] = 1
+                if now - last_update > update_period:
+                    logger.getChild('Counts').debug(counts)
+                    counts = {}
+                    last_update = now
+
+            elif 'subtopic' in item_as_dict.keys():
+                subtopic = item_as_dict.pop('subtopic')
+                topic = f"{config.mqtt.TOPIC_ROOT}/update/{subtopic}"
+                mqtt_client.publish(
+                    topic=topic,
+                    payload=json.dumps(item_as_dict),
+                    qos=0,
+                    retain=False,
+                    properties=None
+                )
+
+            else:
+                logger.error(
+                    f"Unrecognized JSON for MQTT routing: '{item_json}'")
 
         return outbound_pipe_reader
 
+
+    # Format log record locally so that fields are distinct
+    mqtt_formatter = Formatter(fmt=config.logging.formatters.MQTT)
+
     def create_pipe_reader_log_record() -> Callable:
 
         def outbound_pipe_reader():
@@ -278,10 +302,11 @@ def run_mqtt_outbound(config: pyDE1.config.Config,
             nonlocal outbound_pipe, mqtt_client
 
             record: logging.LogRecord = outbound_pipe.recv()
+            formatted = mqtt_formatter.format(record)
 
             mqtt_client.publish(
                 topic=f"{config.mqtt.TOPIC_ROOT}/log",
-                payload=record.getMessage(),
+                payload=formatted,
                 qos=0,
                 retain=False,
                 properties=None
