@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021, 2023 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -14,8 +14,7 @@ import aiosqlite
 
 import pyDE1
 from pyDE1.de1.profile import Profile
-from pyDE1.event_manager import SequencerGateName
-from pyDE1.event_manager.payloads import EventNotificationAction
+from pyDE1.event_manager.payloads import EventNotificationAction, SequencerGateName
 
 logger = pyDE1.getLogger('Database.Insert')
 
@@ -253,6 +252,46 @@ async def connectivity_change(notification: dict,
     await cur.execute(sql, notification)
 
 
+async def device_availability(notification: dict,
+                              sequence_id: str,
+                              cur: aiosqlite.Cursor):
+    sql = "INSERT INTO device_availability " \
+          "(sequence_id, version, sender, arrival_time, create_time, " \
+          "event_time, state, id, name, role) VALUES " \
+          "(:sequence_id, :version, :sender, :arrival_time, :create_time, " \
+          ":event_time, :state, :id, :name, :role)"
+    notification['sequence_id'] = sequence_id
+    await cur.execute(sql, notification)
+
+
+async def scale_change(notification: dict,
+                       sequence_id: str,
+                       cur: aiosqlite.Cursor):
+    sql = "INSERT INTO scale_change " \
+          "(sequence_id, version, sender, arrival_time, create_time, " \
+          "event_time, state, id, name) VALUES " \
+          "(:sequence_id, :version, :sender, :arrival_time, :create_time, " \
+          ":event_time, :state, :id, :name)"
+    notification['sequence_id'] = sequence_id
+    await cur.execute(sql, notification)
+
+
+async def bluedot_update(notification: dict,
+                       sequence_id: str,
+                       cur: aiosqlite.Cursor):
+    sql = "INSERT INTO bluedot_update " \
+          "(sequence_id, version, sender, " \
+          "arrival_time, create_time, event_time, " \
+          "temperature, high_alarm, units, alarm_byte, name) " \
+          "VALUES " \
+          "(:sequence_id, :version, :sender, " \
+          ":arrival_time, :create_time, :event_time, " \
+          ":temperature, :high_alarm, :units, :alarm_byte, :name)"
+    notification['sequence_id'] = sequence_id
+    await cur.execute(sql, notification)
+
+
+
 CLASS_NAME_TO_METHOD = {
     'ShotSampleWithVolumesUpdate': shot_sample_with_volume_update,
     'WeightAndFlowUpdate': weight_and_flow_update,
@@ -264,4 +303,7 @@ CLASS_NAME_TO_METHOD = {
     'AutoTareNotification': auto_tare_notification,
     'ScaleButtonPress': scale_button_press,
     'ConnectivityChange': connectivity_change,
+    'DeviceAvailability': device_availability,
+    'ScaleChange': scale_change,
+    'BlueDOTUpdate': bluedot_update,
 }
\ No newline at end of file
