@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021, 2023 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -9,6 +9,7 @@ SPDX-License-Identifier: GPL-3.0-only
 import asyncio
 import json
 import multiprocessing
+import pprint
 import queue
 import threading
 import time
@@ -28,7 +29,7 @@ from pyDE1.database.recorder_control import RecorderControl
 # from pyDE1.dispatcher.dispatcher import QUEUE_TOO_DEEP
 QUEUE_TOO_DEEP = 1
 
-from pyDE1.event_manager import SequencerGateName
+from pyDE1.event_manager.event_manager import SequencerGateName
 from pyDE1.event_manager.payloads import EventNotificationAction
 from pyDE1.exceptions import DE1TypeError
 
@@ -89,16 +90,25 @@ ROLLING_BUFFER_SIZE = {
     'AutoTareNotification': 3,
     'ScaleButtonPress': 3,
     'ConnectivityChange': 8,            # 2 * 4 from disconnected to ready
+    'DeviceAvailability': 12,           # 3 * 4 from disconnected to ready
+    'ScaleChange': 2,
+    'BlueDOTUpdate': 10,                # At least 10 seconds
 }
-# Total, 59 potential
+# Total, 59 potential (93 now, as of 2.0.0 development)
 
 # If present, only capture (one before and?) all after the sequence begins
+# As of 2.0.0 development, this seems unused
 ROLLING_BUFFER_TIME_LIMITED = [
     'ShotSampleWithVolumesUpdate',
     'WeightAndFlowUpdate',
     'SequencerGateNotification',
 ]
 
+DO_NOT_PERSIST = (
+    'FirmwareUpload',
+    'ScanResults',
+)
+
 # This is probably superfluous, but safer
 rolling_buffers_lock = threading.Lock()
 
@@ -120,12 +130,20 @@ async def dump_rolling_buffers_to_database(rolling_buffers: Dict[str, Deque],
                         and notification['sequence_id'] != sequence_id:
                     pass
                 else:
-                    await db_insert.dict_notification_cursor_only(
-                        notification=notification,
-                        sequence_id=sequence_id,
-                        cur=cur
-                    )
-                    count += 1
+                    try:
+                        await db_insert.dict_notification_cursor_only(
+                            notification=notification,
+                            sequence_id=sequence_id,
+                            cur=cur
+                        )
+                        count += 1
+                    except AttributeError as e:
+                        # There were some strange "no state" errors during dev
+                        # that have not been seen since
+                        logger.exception(e)
+                        logger.info(
+                            f"notification:\n{pprint.pformat(notification)}")
+                        stop_here = notification
         await db.commit()
     t1 = time.time()
     logger.info(f"Dump of {count} notifications in {(t1-t0)*1000:.3f} ms")
@@ -178,8 +196,9 @@ async def record_data(incoming: multiprocessing.Queue):
                             rolling_buffers[data_dict['class']].append(
                                 data_dict)
                     except KeyError:
-                        logger.info("No rolling buffer for "
-                                    f"{data_dict['class']}")
+                        if data_dict['class'] not in DO_NOT_PERSIST:
+                            logger.info("No rolling buffer for "
+                                        f"{data_dict['class']}")
                     pass
 
                     if recording or not consider_sequence_complete.is_set():
