@@ -1,5 +1,5 @@
 """
-Copyright © 2021-2022 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021-2023 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -29,7 +29,16 @@ from pyDE1.dispatcher.resource import (
     Resource, RESOURCE_VERSION, DE1ModeEnum, ConnectivityEnum
 )
 
-MAPPING_VERSION = "5.0.0"
+from pyDE1.event_manager.events import DeviceRole
+from pyDE1.bledev.managed_bleak_device import CaptureRequest
+
+
+try:
+    from pyDE1.source_data import source_data
+except ImportError:
+    source_data = None
+
+MAPPING_VERSION = "7.0.0"
 
 logger = pyDE1.getLogger('Inbound.Mapping')
 
@@ -45,10 +54,11 @@ logger = pyDE1.getLogger('Inbound.Mapping')
 
 class TO (Enum):
     DE1 = auto()
-    DiscoveredDevices = auto()
     FlowSequencer = auto()
     Scale = auto()
     ScaleProcessor = auto()
+    Thermometer = auto()
+    Scanner = auto()
     
 
 class IsAt(NamedTuple):
@@ -63,9 +73,8 @@ class IsAt(NamedTuple):
                                         # path of setter relative to target
     read_only: Optional[bool] = False
     # write_only: attr_path=None and setter_path=setter_attribute_name
-    # internal_type used for conversion to enum
-    internal_type: Optional[Union[type(DE1ModeEnum),
-                                  type(ConnectivityEnum)]] = None
+    # internal_type for conversion to specified enum from JSON value
+    internal_type: Optional[type(Enum)] = None
     # A handful of things, such as connectivity, don't need "ready"
     if_not_ready: bool = False
 
@@ -184,7 +193,8 @@ MAPPING[Resource.VERSION] = {
         'releaselevel': sys.version_info.releaselevel,
         'serial': sys.version_info.serial,
     },
-    'module_versions': module_versions()
+    'module_versions': module_versions(),
+    'source_data': source_data,
 }
 
 # "Specials" -- content-only
@@ -207,19 +217,17 @@ MAPPING[Resource.DE1_FIRMWARE_CANCEL] = IsAt(target=TO.DE1, attr_path=None,
 
 # TODO: How to reference a module-level subroutine? Use module as target?
 
-MAPPING[Resource.SCAN] = {
-    # Passing a number sets the timeout. Passing null takes the default.
-    'begin': IsAt(target=None, attr_path=None,
-                  setter_path='scan_from_api',
-                  v_type=Optional[Union[int, float]])
-}
+MAPPING[Resource.SCAN] = IsAt(target=TO.Scanner, attr_path=None,
+                              setter_path='scan_from_api',
+                              v_type=str,
+                              internal_type=DeviceRole)
 
 # Note: This is an async getter because of the lock
-MAPPING[Resource.SCAN_DEVICES] = {
-    'devices': IsAt(target=TO.DiscoveredDevices, attr_path='devices_for_json',
-                    read_only=True,
-                    v_type=dict)
-}
+# MAPPING[Resource.SCAN_DEVICES] = {
+#     'devices': IsAt(target=TO.DiscoveredDevices, attr_path='devices_for_json',
+#                     read_only=True,
+#                     v_type=dict)
+# }
 
 # Work from leaves back, so can be "included" by reference
 
@@ -244,7 +252,10 @@ MAPPING[Resource.DE1_MODE] = {
 MAPPING[Resource.DE1_STATE] = {
     'state': IsAt(target=TO.DE1, attr_path='state_getter',
                   read_only=True,
-                  v_type=str, internal_type=str),
+                  v_type=str),
+    'mqtt':  IsAt(target=TO.DE1, attr_path='state_update_last_sent',
+                  read_only=True,
+                  v_type=str)
 }
 
 MAPPING[Resource.DE1_FEATURE_FLAGS] = {
@@ -268,6 +279,16 @@ MAPPING[Resource.DE1_FIRMWARE_ID] = {
                attr_path='', v_type=int),
 }
 
+MAPPING[Resource.DE1_AVAILABILITY] = {
+    'mode': IsAt(target=TO.DE1, attr_path='availability_state',
+                 setter_path='availability_setter', v_type=str,
+                 internal_type=CaptureRequest,
+                 if_not_ready=True),
+    'mqtt': IsAt(target=TO.DE1,
+                 attr_path='device_availability_last_sent',
+                 read_only=True, if_not_ready=True, v_type=str),
+}
+
 MAPPING[Resource.DE1_CONNECTIVITY] = {
     'mode': IsAt(target=TO.DE1, attr_path='connectivity',
                  setter_path='connectivity_setter', v_type=str,
@@ -394,7 +415,7 @@ MAPPING[Resource.DE1_SETTING_BEFORE_FLOW] = {
 }
 
 MAPPING[Resource.DE1_SETTING_STEAM] = {
-    'temperature': IsAt(target=ShotSettings, attr_path='TargetSteamTemp', v_type=int),
+    'temperature': IsAt(target=ShotSettings, attr_path='TargetSteamTemp', v_type=Optional[int]),
     'flow': IsAt(target=MMR0x80LowAddr.TARGET_STEAM_FLOW, attr_path='', v_type=float),
     'high_flow_time': IsAt(target=MMR0x80LowAddr.STEAM_START_SECS, attr_path='', v_type=float),
     'purge_deferred': IsAt(target=MMR0x80LowAddr.STEAM_PURGE_MODE, attr_path='', v_type=bool),
@@ -491,6 +512,17 @@ MAPPING[Resource.SCALE_ID] = {
                  if_not_ready=True),
 }
 
+MAPPING[Resource.SCALE_AVAILABILITY] = {
+    'mode': IsAt(target=TO.ScaleProcessor,
+                 attr_path='scale_availability_state',
+                 setter_path="scale_availability_setter", v_type=str,
+                 internal_type=CaptureRequest,
+                 if_not_ready=True),
+    'mqtt': IsAt(target=TO.ScaleProcessor,
+                 attr_path='device_availability_last_sent',
+                 read_only=True, if_not_ready=True, v_type=str),
+}
+
 MAPPING[Resource.SCALE_CONNECTIVITY] = {
     'mode': IsAt(target=TO.ScaleProcessor, attr_path='scale_connectivity',
                  setter_path="connectivity_setter", v_type=str,
@@ -542,6 +574,7 @@ MAPPING[Resource.DE1] = {
     # profiles
     # firmware
     # firmwares
+    'availability': MAPPING[Resource.DE1_AVAILABILITY],
     'connectivity': MAPPING[Resource.DE1_CONNECTIVITY],
     'control': MAPPING[Resource.DE1_CONTROL],
     'setting': MAPPING[Resource.DE1_SETTING],
@@ -552,6 +585,7 @@ MAPPING[Resource.DE1] = {
 
 MAPPING[Resource.SCALE] = {
     'id': MAPPING[Resource.SCALE_ID],
+    'availability': MAPPING[Resource.SCALE_AVAILABILITY],
     'connectivity': MAPPING[Resource.SCALE_CONNECTIVITY],
     'tare': MAPPING[Resource.SCALE_TARE],
     'display': MAPPING[Resource.SCALE_DISPLAY],
@@ -565,3 +599,31 @@ MAPPING[Resource.FLOW_SEQUENCER] = {
     'setting': MAPPING[Resource.FLOW_SEQUENCER_SETTING],
     'parameter_set': MAPPING[Resource.FLOW_SEQUENCER_PARAMETER_SET],
 }
+
+MAPPING[Resource.THERMOMETER_ID] = {
+    'name': IsAt(target=TO.Thermometer, attr_path='name',
+                 v_type=Optional[str],
+                 read_only=True,
+                 if_not_ready=True),
+    # The ID value 'scan' will perform a scan and capture the first found
+    'id': IsAt(target=TO.Thermometer, attr_path='address',
+               v_type=Optional[str],
+               setter_path='change_to_id',
+               if_not_ready=True),
+}
+
+MAPPING[Resource.THERMOMETER_AVAILABILITY] = {
+    'mode': IsAt(target=TO.Thermometer,
+                 attr_path='availability_state',
+                 setter_path="availability_setter", v_type=str,
+                 internal_type=CaptureRequest,
+                 if_not_ready=True),
+    'mqtt': IsAt(target=TO.Thermometer,
+                 attr_path='device_availability_last_sent',
+                 read_only=True, if_not_ready=True, v_type=str),
+}
+
+MAPPING[Resource.THERMOMETER] = {
+    'id': MAPPING[Resource.THERMOMETER_ID],
+    'availability': MAPPING[Resource.THERMOMETER_AVAILABILITY],
+}
\ No newline at end of file
