@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021, 2023 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -8,10 +8,15 @@ SPDX-License-Identifier: GPL-3.0-only
 Common events for scales
 """
 
-from pyDE1.event_manager import EventPayload
+from pyDE1.event_manager.payloads import EventPayload
+from pyDE1.event_manager.events import DeviceAvailability
 
 
 class ScaleWeightUpdate(EventPayload):
+    """
+    See WeightAndFlowUpdate for API-visible class
+    """
+    _internal_only = True
 
     def __init__(self,
                  arrival_time: float,
@@ -22,6 +27,7 @@ class ScaleWeightUpdate(EventPayload):
         self.scale_time = scale_time
         self.weight = weight
 
+
 class ScaleButtonPress(EventPayload):
 
     def __init__(self, arrival_time: float, button: int):
@@ -65,3 +71,13 @@ class WeightAndFlowUpdate(EventPayload):
         self.median_weight_time: float = median_weight_time
         self.median_flow: float = median_flow
         self.median_flow_time: float = median_flow_time
+
+
+class ScaleChange(DeviceAvailability):
+    """
+    Gets set when the address of the scale changes "behind the scenes"
+    such as with a call to scale.take_over_from()
+
+    Not sent on initialization at this time
+    """
+    pass
