@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021-2023 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -9,13 +9,17 @@ SPDX-License-Identifier: GPL-3.0-only
 import asyncio
 import enum
 import logging
+import multiprocessing
+import queue
 import re
+import sys
+import time
 import uuid
 
 from datetime import datetime
-from typing import Optional
+from typing import Optional, Union
 
-logger_cancel_tasks = logging.getLogger('CancelTasks')
+logger_cancel_tasks = logging.getLogger('Util.CancelTasks')
 
 
 def task_name_exists(name_to_find: str, starts_with=False):
@@ -60,6 +64,22 @@ def address_is_persistent(address: str) -> bool:
         return False
 
 
+if sys.version_info.major >= 3 and sys.version_info.minor >= 11:
+    # In 3.11, the behavior of enum.IntFlag changed
+    # https://docs.python.org/3.11/whatsnew/3.11.html
+    # We're trying to get a human-readable version
+    def enum_intflag_for_json(val: enum.IntEnum):
+        if isinstance(name := val.name, str) and len(name):
+            retval = name
+        else:
+            retval = str(val.value)
+        return retval
+else:
+    # Remove the class name
+    def enum_intflag_for_json(val: enum.IntFlag):
+        return str(val).split('.', 2)[1]
+
+
 def prep_for_json(val):
     """
     Special cases for conversion to JSON:
@@ -74,8 +94,7 @@ def prep_for_json(val):
     if val is None or isinstance(val, (float, str, bool)):
         return val
     elif isinstance(val, enum.IntFlag):
-        # Remove the class name
-        return str(val).split('.',2)[1]
+        return enum_intflag_for_json(val)
     elif isinstance(val, enum.IntEnum):
         return val.name
     elif isinstance(val, enum.Enum):
@@ -117,7 +136,10 @@ def data_as_readable(data, replace_rnt=True, replace_space=False):
     :return:                str: converted to readable, or ''
     """
     if isinstance(data, (bytes, bytearray)):
-        data = data.decode('ascii')  # enforce one character per byte
+        try:
+            data = data.decode('ascii')  # enforce one character per byte
+        except UnicodeDecodeError:
+            return ''
     if not isinstance(data, str):
         raise TypeError("Expected str, bytes, or bytearray")
     out = ""
@@ -167,3 +189,93 @@ def timestamp_to_str_with_ms(timestamp: float, show_date=True) -> str:
         (d, t) = string.split(' ', maxsplit=2)
         string = t
     return string
+
+
+def call_str(full_trace=True) -> str:
+    import inspect
+    stack = inspect.stack()[2]
+    retval = f"at {stack.function}:{stack.lineno}"
+    if full_trace:
+        idx = 3
+        while True:
+            try:
+                stack = inspect.stack()[idx]
+                next_caller = f"{stack.function}:{stack.lineno}"
+                if stack.function.startswith('_run'):
+                    break
+                retval = f"{retval} < {next_caller}"
+            except IndexError:
+                break
+            idx += 1
+    return retval
+
+
+# As run in a thread-pool executor, need a way to cleanly stop the thread
+# so require abandon_on_event
+
+async def mp_event_wait(event: multiprocessing.Event,
+                        timeout: Optional[float],
+                        abandon_on_event:
+                            Union[asyncio.Event,
+                                  multiprocessing.Event]) -> bool:
+    done = event.is_set()
+    end_time = (time.time() + timeout) if timeout else None
+    RECHECK_PERIOD = 1.0 # seconds
+    while (not done
+           and (not end_time
+                or (end_time and (now := time.time()) < end_time))
+           and (abandon_on_event and not abandon_on_event.is_set())):
+        if end_time:
+            wait_time = min(end_time - now, RECHECK_PERIOD)
+        else:
+            wait_time = RECHECK_PERIOD
+        task = asyncio.get_running_loop().run_in_executor(
+            None, event.wait, wait_time)
+        done = await task
+    return done
+
+
+async def mq_queue_get(mp_queue: multiprocessing.Queue,
+                       timeout: Optional[float],
+                       abandon_on_event: Union[asyncio.Event,
+                                               multiprocessing.Event]):
+    end_time = (time.time() + timeout) if timeout else None
+    RECHECK_PERIOD = 1.0  # seconds
+    done = False
+    qexc = None
+    data = None
+    while (not done
+           and (not end_time
+                or (end_time and (now := time.time()) < end_time))
+           and (abandon_on_event and not abandon_on_event.is_set())):
+        qexc = None
+        if end_time:
+            wait_time = min(end_time - now, RECHECK_PERIOD)
+        else:
+            wait_time = RECHECK_PERIOD
+        try:
+            task = asyncio.get_running_loop().run_in_executor(
+                None, mp_queue.get, True, wait_time)
+            data = await task
+            done = True  # As otherwise it would have raised queue.Empty
+        except queue.Empty as e:
+            qexc = e
+            done = False
+    if qexc and not abandon_on_event.is_set():
+        raise qexc
+    else:
+        return data
+
+
+class EventReadOnly:
+    """
+    Like an asyncio.Event, but can only wait() and check is_set()
+    """
+    def __init__(self, backing_event: asyncio.Event):
+        self._backing_event = backing_event
+
+    def is_set(self):
+        return self._backing_event.is_set()
+
+    async def wait(self):
+        return await self._backing_event.wait()
